"""Models used by the client"""
import datetime
import json
import typing
from typing import Optional, TypeVar

import pydantic
from pydantic import validator

NameType = TypeVar('NameType')


def to_lower_camel(s: str) -> str:
    """Alias generator to avoid breaking PEP8"""
    first, *others = s.split('_')
    return ''.join([first.lower(), *map(str.capitalize, others)])


class BaseModel(pydantic.BaseModel):
    """Expanded BaseModel for convenience"""

    @classmethod
    def from_dict(cls, data: dict):
        """Initialize class from dict"""
        return cls(**data)

    @classmethod
    def from_json(cls, json_data: str):
        """Initialize class from json file"""
        data = json.loads(json_data)
        return cls.from_dict(data)


class Batch(BaseModel):
    """Model for making json formatted list of a Batch"""
    client: str
    batchid: str
    period: str
    interface: str
    vouchertype: str
    batchid_interface: str


class Voucher(BaseModel):
    voucherdate: Optional[datetime.date]
    exref: str
    voucherno: int


class Transaction(BaseModel):
    account: str
    amount: float
    transdate: datetime.date
    curamount: float
    currency: str
    description: str
    dim1: str
    dim2: str
    dim3: str
    dim4: str
    dim5: str
    dim6: str
    dim7: str
    sequenceno: int
    taxcode: str
    transtype: str
    arrivaldate: datetime.date
    compldelay: datetime.date
    discdate: datetime.date
    duedate: datetime.date
    extinvref: str
    voucherno_interface: str


class Multi(BaseModel):
    batch: Batch
    vouchers: typing.List[Voucher]
    transactions: typing.List[Transaction]

